/////////////////////
// MCMCpack README //
/////////////////////

// Authors

Andrew D. Martin <admart@umich.edu>
Kevin M. Quinn <kmq@umich.edu>
Jong Hee Park <jongheepark@snu.ac.kr>

// Compilation

This package (along with Scythe) uses C++ and the Standard Template
Library (STL).  We suggest using of the GCC compiler 4.0 or greater.  The
current package has been tested using GCC 4.0 on Linux and MacOS X. 

Many thanks to Dan Pemstein for helping with all sorts of C++ issues,
and to Kurt Hornik and Fritz Leisch for their help with debugging as
well as their service to the R community.  We are also very grateful to
Brian Ripley who provided C++ patches to fix a number of clang and Solaris
issues. 

// Acknowledgments

We gratefully acknowledge support from:

* National Science Foundation, Program in Methodology, Measurement, and 
Statistics, Grants SES-0350646 and SES-0350613

* Washington University, Department of Political Science, the
Weidenbaum Center on the Economy, Government, and Public Policy
(http://wc.wustl.edu), and the Center for Empirical Research in the Law
(http://cerl.wustl.edu)

* Harvard University, Department of Government and the
Institute for Quantitative Social Sciences (http://iq.harvard.edu)

Neither the National Science Foundation, Washington University, or
Harvard University bear any responsibility for the content of this
package.

Please contact Jong Hee Park <jongheepark@snu.ac.kr> if
you have any problems or questions.

--
Jong Hee Park, Ph.D.
Professor in Dept. Political Science and International Relations
Seoul National University

Email: jongheepark@snu.ac.kr
WWW: http://jhp.snu.ac.kr

